package org.expressolivre.cte.pages.email;

import org.expressolivre.cte.pages.common.Page;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.FindBy;

/**
 * Pagina de administracao de filtros.
 * 
 * @author L.F.Estivalet (Serpro)
 * 
 *         Created on Jun 16, 2011 at 3:47:18 PM
 * 
 */
public class MailFilterPage extends Page {

	@FindBy(xpath = "//div[@id='window_ffilter_ccform']/div[@id='form_body']/a[1]")
	private WebElement filterNewRule;

	@FindBy(xpath = "//div[@id='window_ffilter_ccform']/div[@id='form_body']/a[2]")
	private WebElement filterNewOutOfOfficeRule;

	@FindBy(id = "field8")
	private WebElement filterOutOfOfficeText;

	@FindBy(xpath = "//div[@id='window_ffilter_ccform']/div[@id='form_body']/span[2]/input")
	private WebElement filterSaveRules;

	@FindBy(id = "field1")
	private WebElement filterRecepient;

	@FindBy(id = "field2")
	private WebElement filterTo;

	@FindBy(id = "field3")
	private WebElement filterSubject;

	@FindBy(id = "field4")
	private WebElement filterSize;

	@FindBy(id = "select_size")
	private WebElement filterSelectSize;

	@FindBy(id = "radio1")
	private WebElement filterArchieveIn;

	@FindBy(id = "select_mailboxes")
	private WebElement filterMailBoxes;

	@FindBy(id = "radio2")
	private WebElement filterForwardMessage;

	@FindBy(id = "radio3")
	private WebElement filterRejectMessage;

	@FindBy(id = "field6")
	private WebElement filterRejectMessageText;

	@FindBy(id = "radio4")
	private WebElement filterDeleteMessage;

	@FindBy(id = "radio5")
	private WebElement filterApplyImportantFilter;

	public MailFilterPage(WebDriver driver) {
		super(driver);
	}

	public void newFilterRule() {
		this.filterNewRule.click();
	}

	public void newFilterOutOfOfficeRule() {
		this.filterNewOutOfOfficeRule.click();
	}

	public void setFilterOutOfOfficeText(String text) {
		this.filterOutOfOfficeText.clear();
		this.filterOutOfOfficeText.sendKeys(text);
	}

	public String getFilterOutOfOfficeText() {
		return this.filterOutOfOfficeText.getValue();
	}

	public void editFilterRule(String id) {
		driver.findElement(
				By.xpath("//div[@id='window_ffilter_ccform']/div[@id='form_body']/div[@id='set_rules']/a["
						+ id + "]")).click();
	}

	public void editOutOfOfficeFilterRule() {
		driver.findElement(By.xpath("//div[@id='set_out']/a/b")).click();
	}

	public String getFilterRuleText(String id) {
		return driver
				.findElement(
						By.xpath("//div[@id='window_ffilter_ccform']/div[@id='form_body']/div[@id='set_rules']/a["
								+ id + "]")).getText();
	}

	public void setFilterRecepient(String value) {
		this.filterRecepient.clear();
		this.filterRecepient.sendKeys(value);
	}

	public String getFilterRecepient() {
		return this.filterRecepient.getValue();
	}

	public void setFilterTo(String value) {
		this.filterTo.clear();
		this.filterTo.sendKeys(value);
	}

	public String getFilterTo() {
		return this.filterTo.getValue();
	}

	public void setFilterSubject(String value) {
		this.filterSubject.clear();
		this.filterSubject.sendKeys(value);
	}

	public String getFilterSubject() {
		return this.filterSubject.getValue();
	}

	public void setFilterSize(String value) {
		this.filterSize.clear();
		this.filterSize.sendKeys(value);
	}

	public String getFilterSize() {
		return this.filterSize.getValue();
	}

	public void setFilterArchieveIn() {
		this.filterArchieveIn.click();
	}

	public WebElement getFilterArchieveIn() {
		return this.filterArchieveIn;
	}

	public void setFilterRejectMessage() {
		this.filterRejectMessage.click();
	}

	public void setFilterRejectMessageText(String message) {
		this.filterRejectMessageText.sendKeys(message);
	}

	public void setFilterMailBox(String mailBox) {
		super.setComboValue(this.filterMailBoxes, mailBox);
	}

	public String getFilterMailBox() {
		return super.getComboValue(this.filterMailBoxes);
	}

	public void saveFilterRules() {
		this.filterSaveRules.click();
	}

	/**
	 * Apaga uma regra do filtro.
	 * 
	 * @param id
	 */
	public void deleteFilterRule(int id) {
		selectFilterRule(id);
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[3]"))
				.click();
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[4]"))
				.click();
	}

	/**
	 * Apaga filtros previamente selecionados.
	 */
	public void deleteFilters() {
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[3]"))
				.click();
		// driver.findElement(By.xpath("//div[@id='form_buttons']/input[4]"))
		// .click();
	}

	/**
	 * Desabilita uma regra do filtro.
	 * 
	 * @param id
	 */
	public void disableFilterRule(int id) {
		selectFilterRule(id);
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[2]"))
				.click();
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[4]"))
				.click();
	}

	/**
	 * Habilita uma regra do filtro.
	 * 
	 * @param id
	 */
	public void enableFilterRule(int id) {
		selectFilterRule(id);
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[1]"))
				.click();
		driver.findElement(By.xpath("//div[@id='form_buttons']/input[4]"))
				.click();
	}

	/**
	 * Marca o checkbox do filtro selecionado.
	 * 
	 * @param id
	 *            Id do filtro selecionado.
	 */
	public void selectFilterRule(int id) {
		this.waitForElement(By.id("rule_" + id));
		driver.findElement(By.id("rule_" + id)).click();
	}

	public void selectOutOfOfficeFilterRule() {
		this.waitForElement(By.id("out_0"));
		driver.findElement(By.id("out_0")).click();
	}

}
