package org.expressolivre.cte.pages.email;

import org.expressolivre.cte.pages.common.Page;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.FindBy;

/**
 * @author L.F.Estivalet (Serpro)
 * 
 *         Created on Feb 3, 2011 at 11:03:45 AM
 * 
 */
public class MailPreferencesPage extends Page {

	/** Utilizar nome (CN) ao invés de login (uid) em pastas compartilhadas? */
	@FindBy(name = "user[uid2cn]")
	private WebElement userCNInSharedFolders;

	/** Você deseja exibir automaticamente o cabecalho da mensagem? */
	@FindBy(name = "user[show_head_msg_full]")
	private WebElement showHeadMsgFull;

	/** Qual o número máximo de mensagens por página? */
	@FindBy(name = "user[max_email_per_page]")
	private WebElement maxEmailPerPage;

	/** Pré-visualizar a mensagem na coluna Assunto? */
	@FindBy(name = "user[preview_msg_subject]")
	private WebElement previewMsgSubject;

	/** Pré-visualizar a mensagem em uma caixa tool-tip? */
	@FindBy(name = "user[preview_msg_tip]")
	private WebElement previewMsgTip;

	/** View extended information about users* */
	@FindBy(name = "user[extended_info]")
	private WebElement extendedInfo;

	/** Salvar mensagens excluídas na pasta Lixeira? */
	@FindBy(name = "user[save_deleted_msg]")
	private WebElement saveDeletedMsg;

	/** Apagar mensagens após quantos dias na lixeira? */
	@FindBy(name = "user[delete_trash_messages_after_n_days]")
	private WebElement deleteTrashMessagesAfterNDays;

	/** Você gostaria de usar mensagens locais? */
	@FindBy(name = "user[use_local_messages]")
	private WebElement useLocalMessages;

	/** Ação desejada ao arquivar mensagem nas pastas locais */
	@FindBy(name = "user[keep_archived_messages]")
	private WebElement keepArchivedMessages;

	/** Criar automáticamente pastas locais padrão? */
	@FindBy(name = "user[auto_create_local]")
	private WebElement autoCreateLocal;

	/** Exibir mensagem anterior, após apagar mensagem atual? */
	@FindBy(name = "user[delete_and_show_previous_message]")
	private WebElement deleteAndShowPreviousMessage;

	/** Você quer receber um alerta para mensagens novas? */
	@FindBy(name = "user[alert_new_msg]")
	private WebElement alertNewMsg;

	/** Exibir resumo do ExpressoMail na tela inicial do Expresso? */
	@FindBy(name = "user[mainscreen_showmail]")
	private WebElement mainScreenShowMail;

	/** Você gostaria de usar a função de desanexar? */
	@FindBy(name = "user[remove_attachments_function]")
	private WebElement removeAttachmentsFunction;

	/** Você gostaria de usar a flag de importante na edição de emails? */
	@FindBy(name = "user[use_important_flag]")
	private WebElement useImportantFlag;

	/** Quer usar o corretor ortográfico no editor de emails? */
	@FindBy(name = "user[use_SpellChecker]")
	private WebElement useSpellChecker;

	/** Salvar automaticamente as mensagens enviadas na pasta */
	@FindBy(name = "user[save_in_folder]")
	private WebElement saveInFolder;

	/**
	 * Trocar remetente pelo destinatário na coluna QUEM somente na pasta
	 * Enviado
	 */
	@FindBy(name = "user[from_to_sent]")
	private WebElement fromToSent;

	/** Ocultar menu das pastas? */
	@FindBy(name = "user[hide_folders]")
	private WebElement hideFolders;

	/** Qual o número máximo de resultados na busca de e-mails? */
	@FindBy(name = "user[search_result_number]")
	private WebElement searchResultNumber;

	/** Qual é o número mínimo de caracteres ao pesquisar contatos? */
	@FindBy(name = "user[search_characters_number]")
	private WebElement searchCharactersNumber;

	/** Qual a altura das linhas na lista de mensagens? */
	@FindBy(name = "user[line_height]")
	private WebElement lineHeight;

	/**
	 * Você quer que se aumente o limite de tamanho de emails que são mostrados?
	 */
	@FindBy(name = "user[max_msg_size]")
	private WebElement maxMsgSize;

	/** Usar contatos dinâmicos? */
	@FindBy(name = "user[use_dynamic_contacts]")
	private WebElement useDynamicContacts;

	/** Usar teclas de atalho? */
	@FindBy(name = "user[use_shortcuts]")
	private WebElement useShortcuts;

	/** Salvar rascunhos automaticamente */
	@FindBy(name = "user[auto_save_draft]")
	private WebElement autoSaveDraft;

	/** Marcar confirmacao leitura automaticamente */
	@FindBy(name = "user[return_recipient_deafault]")
	private WebElement returnRecipientDefault;

	/** Qual o seu fuso-horário? */
	@FindBy(name = "user[timezone]")
	private WebElement timezone;

	/** Aonde a pesquisa rápida deve atuar por padrão? */
	@FindBy(name = "user[quick_search_default]")
	private WebElement quickSearchDefault;

	/** Qual o tamanho máximo das imagens inseridas nos emails? */
	@FindBy(name = "user[image_size]")
	private WebElement imageSize;

	/** Possibilitar assinar/cifrar digitalmente a mensagem? */
	@FindBy(name = "user[use_signature_digital_cripto]")
	private WebElement useSignatureDigitalCripto;

	/** Sempre assinar digitalmente a mensagem? */
	@FindBy(name = "user[use_signature_digital]")
	private WebElement useSignatureDigital;

	/** Sempre cifrar digitalmente a mensagem? */
	@FindBy(name = "user[use_signature_cripto]")
	private WebElement useSignatureCripto;

	/** Inserir assinatura automaticamente em novas mensagens? */
	@FindBy(name = "user[use_signature]")
	private WebElement useSignature;

	/** Tipo de Assinatura */
	@FindBy(name = "user[type_signature]")
	private WebElement typeSignature;

	@FindBy(name = "submit")
	private WebElement savePreferences;

	public MailPreferencesPage(WebDriver driver) {
		super(driver);
		// TODO Auto-generated constructor stub
	}

	public void savePreferences() {
		this.savePreferences.click();
	}

	/**
	 * @param userCNInSharedFolders
	 *            the userCNInSharedFolders to set
	 */
	public void setUserCNInSharedFolders(String userCNInSharedFolders) {
		System.out.println("setting...");
		super.setComboValue(this.userCNInSharedFolders, userCNInSharedFolders);
	}

	/**
	 * @param showHeadMsgFull
	 *            the showHeadMsgFull to set
	 */
	public void setShowHeadMsgFull(String showHeadMsgFull) {
		super.setComboValue(this.showHeadMsgFull, showHeadMsgFull);
	}

	/**
	 * @param maxEmailPerPage
	 *            the maxEmailPerPage to set
	 */
	public void setMaxEmailPerPage(String maxEmailPerPage) {
		super.setComboValue(this.maxEmailPerPage, maxEmailPerPage);
	}

	/**
	 * @param previewMsgSubject
	 *            the previewMsgSubject to set
	 */
	public void setPreviewMsgSubject(String previewMsgSubject) {
		super.setComboValue(this.previewMsgSubject, previewMsgSubject);
	}

	/**
	 * @param previewMsgTip
	 *            the previewMsgTip to set
	 */
	public void setPreviewMsgTip(String previewMsgTip) {
		super.setComboValue(this.previewMsgTip, previewMsgTip);
	}

	/**
	 * @param extendedInfo
	 *            the extendedInfo to set
	 */
	public void setExtendedInfo(String extendedInfo) {
		super.setComboValue(this.extendedInfo, extendedInfo);
	}

	/**
	 * @param saveDeletedMsg
	 *            the saveDeletedMsg to set
	 */
	public void setSaveDeletedMsg(String saveDeletedMsg) {
		super.setComboValue(this.saveDeletedMsg, saveDeletedMsg);
	}

	/**
	 * @param deleteTrashMessagesAfterNDays
	 *            the deleteTrashMessagesAfterNDays to set
	 */
	public void setDeleteTrashMessagesAfterNDays(
			String deleteTrashMessagesAfterNDays) {
		super.setComboValue(this.deleteTrashMessagesAfterNDays,
				deleteTrashMessagesAfterNDays);
	}

	/**
	 * @param useLocalMessages
	 *            the useLocalMessages to set
	 */
	public void setUseLocalMessages(String useLocalMessages) {
		super.setComboValue(this.useLocalMessages, useLocalMessages);
	}

	/**
	 * @param keepArchivedMessages
	 *            the keepArchivedMessages to set
	 */
	public void setKeepArchivedMessages(String keepArchivedMessages) {
		super.setComboValue(this.keepArchivedMessages, keepArchivedMessages);
	}

	/**
	 * @param autoCreateLocal
	 *            the autoCreateLocal to set
	 */
	public void setAutoCreateLocal(String autoCreateLocal) {
		super.setComboValue(this.autoCreateLocal, autoCreateLocal);
	}

	/**
	 * @param deleteAndShowPreviousMessage
	 *            the deleteAndShowPreviousMessage to set
	 */
	public void setDeleteAndShowPreviousMessage(
			String deleteAndShowPreviousMessage) {
		super.setComboValue(this.deleteAndShowPreviousMessage,
				deleteAndShowPreviousMessage);
	}

	/**
	 * @param alertNewMsg
	 *            the alertNewMsg to set
	 */
	public void setAlertNewMsg(String alertNewMsg) {
		super.setComboValue(this.alertNewMsg, alertNewMsg);
	}

	/**
	 * @param mainScreenShowMail
	 *            the mainScreenShowMail to set
	 */
	public void setMainScreenShowMail(String mainScreenShowMail) {
		super.setComboValue(this.mainScreenShowMail, mainScreenShowMail);
	}

	/**
	 * @param removeAttachmentsFunction
	 *            the removeAttachmentsFunction to set
	 */
	public boolean setRemoveAttachmentsFunction(String removeAttachmentsFunction) {
		return super.setComboValue(this.removeAttachmentsFunction,
				removeAttachmentsFunction);
	}

	/**
	 * @param returnRecipientDefault
	 * @return
	 */
	public boolean setReturnRecipientDefault(String returnRecipientDefault) {
		return super.setComboValue(this.returnRecipientDefault,
				returnRecipientDefault);
	}

	/**
	 * @param useImportantFlag
	 *            the useImportantFlag to set
	 */
	public boolean setUseImportantFlag(String useImportantFlag) {
		return super.setComboValue(this.useImportantFlag, useImportantFlag);
	}

	/**
	 * @param useSpellChecker
	 *            the useSpellChecker to set
	 */
	public boolean setUseSpellChecker(String useSpellChecker) {
		return super.setComboValue(this.useSpellChecker, useSpellChecker);
	}

	/**
	 * @param saveInFolder
	 *            the saveInFolder to set
	 */
	public void setSaveInFolder(String saveInFolder) {
		super.setComboValue(this.saveInFolder, saveInFolder);
	}

	/**
	 * @param fromToSent
	 *            the fromToSent to set
	 */
	public void setFromToSent(String fromToSent) {
		super.setComboValue(this.fromToSent, fromToSent);
	}

	/**
	 * @param hideFolders
	 *            the hideFolders to set
	 */
	public void setHideFolders(String hideFolders) {
		super.setComboValue(this.hideFolders, hideFolders);
	}

	/**
	 * @param searchResultNumber
	 *            the searchResultNumber to set
	 */
	public void setSearchResultNumber(String searchResultNumber) {
		super.setComboValue(this.searchResultNumber, searchResultNumber);
	}

	/**
	 * @param searchCharactersNumber
	 *            the searchCharactersNumber to set
	 */
	public void setSearchCharactersNumber(String searchCharactersNumber) {
		super.setComboValue(this.searchCharactersNumber, searchCharactersNumber);
	}

	/**
	 * @param lineHeight
	 *            the lineHeight to set
	 */
	public void setLineHeight(String lineHeight) {
		super.setComboValue(this.lineHeight, lineHeight);
	}

	/**
	 * @param maxMsgSize
	 *            the maxMsgSize to set
	 */
	public void setMaxMsgSize(String maxMsgSize) {
		super.setComboValue(this.maxMsgSize, maxMsgSize);
	}

	/**
	 * @param useDynamicContacts
	 *            the useDynamicContacts to set
	 */
	public void setUseDynamicContacts(String useDynamicContacts) {
		super.setComboValue(this.useDynamicContacts, useDynamicContacts);
	}

	/**
	 * @param useShortcuts
	 *            the useShortcuts to set
	 */
	public void setUseShortcuts(String useShortcuts) {
		super.setComboValue(this.useShortcuts, useShortcuts);
	}

	/**
	 * @param autoSaveDraft
	 *            the autoSaveDraft to set
	 */
	public void setAutoSaveDraft(String autoSaveDraft) {
		super.setComboValue(this.autoSaveDraft, autoSaveDraft);
	}

	/**
	 * @param timezone
	 *            the timezone to set
	 */
	public void setTimezone(String timezone) {
		super.setComboValue(this.timezone, timezone);
	}

	/**
	 * @param quickSearchDefault
	 *            the quickSearchDefault to set
	 */
	public void setQuickSearchDefault(String quickSearchDefault) {
		super.setComboValue(this.quickSearchDefault, quickSearchDefault);
	}

	/**
	 * @param imageSize
	 *            the imageSize to set
	 */
	public void setImageSize(String imageSize) {
		super.setComboValue(this.imageSize, imageSize);
	}

	/**
	 * @param useSignatureDigitalCripto
	 *            the useSignatureDigitalCripto to set
	 */
	public void setUseSignatureDigitalCripto(String useSignatureDigitalCripto) {
		super.setComboValue(this.useSignatureDigitalCripto,
				useSignatureDigitalCripto);
	}

	/**
	 * @param useSignatureDigital
	 *            the useSignatureDigital to set
	 */
	public void setUseSignatureDigital(String useSignatureDigital) {
		super.setComboValue(this.useSignatureDigital, useSignatureDigital);
	}

	/**
	 * @param useSignatureCripto
	 *            the useSignatureCripto to set
	 */
	public void setUseSignatureCripto(String useSignatureCripto) {
		super.setComboValue(this.useSignatureCripto, useSignatureCripto);
	}

	/**
	 * @param useSignature
	 *            the useSignature to set
	 */
	public void setUseSignature(String useSignature) {
		super.setComboValue(this.useSignature, useSignature);
	}

	/**
	 * @param typeSignature
	 *            the typeSignature to set
	 */
	public void setTypeSignature(String typeSignature) {
		super.setComboValue(this.typeSignature, typeSignature);
	}

	/**
	 * @param savePreferences
	 *            the savePreferences to set
	 */
	public void setSavePreferences(String savePreferences) {
		super.setComboValue(this.savePreferences, savePreferences);
	}

}
